# from django.apps import apps as django_apps
# from django.test import TestCase
# from edc_lab.models import Aliquot, Box, BoxType
# from edc_lab.models.box_item import BoxItem
#
# from edc_lab_dashboard.model_wrappers import BoxModelWrapper
#
# app_config = django_apps.get_app_config("edc_lab_dashboard")
#
#
# class TestModelWrapper(TestCase):
#     def setUp(self):
#         self.box_type = BoxType.objects.create(name="9 x 9", across=9, down=9, total=81)
#         self.box = Box.objects.create(box_identifier="12345678", box_type=self.box_type)
#         self.box_item = BoxItem.objects.create(box=self.box, position=0)
#         self.aliquot = Aliquot.objects.create(
#             subject_identifier="ABCDEFG",
#             count=1,
#             is_primary=True,
#             aliquot_type="Whole Blood",
#             numeric_code="02",
#             alpha_code="WB",
#         )
#
#         self.wrapper_cls = BoxModelWrapper
#         box_type = BoxType.objects.create(across=9, down=9, total=81)
#         self.box = Box.objects.create(box_identifier="1234", box_type=box_type)
#
#     def test_box_model_wrapper_href(self):
#         wrapper = self.wrapper_cls(self.box)
#         self.assertEqual(
#             wrapper.href,
#             f"/admin/edc_lab/box/{self.box.id}/change/?"
#             f"next=edc_lab_dashboard:pack_listboard_url&",
#         )
#
#     def test_box_model_wrapper_template_attrs(self):
#         wrapper = self.wrapper_cls(self.box)
#         attrs = ["human_readable_identifier", "comment", "created", "user_created"]
#         for attr in attrs:
#             with self.subTest(attr=attr):
#                 self.assertTrue(hasattr(wrapper, attr))



# from copy import copy
#
# from django.apps import apps as django_apps
# from django.test import TestCase
# from edc_lab.models import Aliquot, Box, BoxItem, BoxType
#
# from edc_lab_dashboard.model_wrappers import AliquotModelWrapper
#
# app_config = django_apps.get_app_config("edc_lab_dashboard")
#
#
# class TestModelWrapper(TestCase):
#     def setUp(self):
#         self.box_type = BoxType.objects.create(name="9 x 9", across=9, down=9, total=81)
#         self.box = Box.objects.create(box_identifier="12345678", box_type=self.box_type)
#         self.box_item = BoxItem.objects.create(box=self.box, position=0)
#         self.aliquot = Aliquot.objects.create(
#             subject_identifier="ABCDEFG",
#             count=1,
#             is_primary=True,
#             aliquot_type="Whole Blood",
#             numeric_code="02",
#             alpha_code="WB",
#         )
#         self.wrapper_cls = AliquotModelWrapper
#
#     def test_aliquot_model_wrapper(self):
#         wrapper = self.wrapper_cls(self.aliquot)
#         self.assertEqual(
#             wrapper.href,
#             f"/admin/edc_lab/aliquot/{self.aliquot.id}/change/?"
#             f"next=edc_lab_dashboard:aliquot_listboard_url&",
#         )
#         self.assertIn("edc_lab_dashboard:aliquot_listboard_url", wrapper.href)
#
#     def test_aliquot_wrapper_attrs(self):
#         """Asserts attrs used in template exist."""
#         wrapper = self.wrapper_cls(self.aliquot)
#         attrs = [
#             "is_primary",
#             "box_item",
#             "manifest_item",
#             "subject_identifier",
#             "aliquot_datetime",
#             "aliquot_type",
#             "numeric_code",
#             "alpha_code",
#             "box_item",
#             "shipped",
#             "count",
#         ]
#         for attr in attrs:
#             with self.subTest(attr=attr):
#                 self.assertTrue(hasattr(wrapper, attr))
#
#     def test_aliquot_wrapper_attrs2(self):
#         wrapper = self.wrapper_cls(self.aliquot)
#         self.assertTrue(wrapper.human_readable_identifier)
#         self.assertTrue(wrapper.is_primary)
#         self.assertTrue(wrapper.box_item)
#         self.assertTrue(wrapper.box_item.box.box_identifier)
#         self.assertTrue(wrapper.box_item.box.get_category_display)
#         self.assertTrue(wrapper.box_item.box.box_identifier)
#         self.assertTrue(wrapper.subject_identifier)
#         self.assertTrue(wrapper.aliquot_datetime)
#         self.assertTrue(wrapper.aliquot_type)
#         self.assertTrue(wrapper.numeric_code)
#         self.assertTrue(wrapper.alpha_code)
#         self.assertTrue(wrapper.box_item)
#         self.assertTrue(wrapper.count)
#         self.assertIsNone(wrapper.manifest_item)
#         self.assertFalse(wrapper.shipped)
#
#     def test_aliquot_wrapper_attrs3(self):
#         aliquot = copy(self.aliquot)
#         wrapper = self.wrapper_cls(aliquot)
#         self.box_item.delete()
#         self.assertIsNone(wrapper.box_item)
#
#         aliquot = copy(self.aliquot)
#         wrapper = self.wrapper_cls(aliquot)
#         self.assertIsNone(wrapper.box_item)



# from django.contrib.sites.models import Site
# from django.test import TestCase
# from django.urls.base import reverse
# from edc_lab.models import (
#     Aliquot,
#     Box,
#     BoxItem,
#     BoxType,
#     Consignee,
#     Manifest,
#     ManifestItem,
#     Shipper,
# )
#
# from edc_lab_dashboard.model_wrappers import (
#     BoxModelWrapper,
#     ManageBoxItemModelWrapper,
#     ManifestItemModelWrapper,
#     ManifestModelWrapper,
# )
#
#
# class TestModelWrapper(TestCase):
#     def setUp(self):
#         self.box_type = BoxType.objects.create(name="9 x 9", across=9, down=9, total=81)
#         self.box = Box.objects.create(box_identifier="12345678", box_type=self.box_type)
#         self.box_item = BoxItem.objects.create(box=self.box, position=0)
#         self.aliquot = Aliquot.objects.create(
#             subject_identifier="ABCDEFG",
#             count=1,
#             is_primary=True,
#             aliquot_type="Whole Blood",
#             numeric_code="02",
#             alpha_code="WB",
#         )
#
#     def test_box_model_wrapper(self):
#         wrapper_cls = BoxModelWrapper
#         box_type = BoxType.objects.create(across=9, down=9, total=81)
#         obj = Box.objects.create(box_identifier="1234", box_type=box_type)
#         wrapper = wrapper_cls(obj)
#         self.assertEqual(
#             wrapper.href,
#             f"/admin/edc_lab/box/{obj.id}/change/?"
#             f"next=edc_lab_dashboard:pack_listboard_url&",
#         )
#         self.assertEqual(
#             reverse("edc_lab_dashboard:pack_listboard_url"),
#             "/edc_lab_dashboard/listboard/pack/",
#         )
#
#     def test_manage_box_item_model_wrapper(self):
#         wrapper_cls = ManageBoxItemModelWrapper
#         # usually this will come from app_config
#         box_type = BoxType.objects.create(across=9, down=9, total=81)
#         box_identifier = "1234"
#         box = Box.objects.create(box_identifier=box_identifier, box_type=box_type)
#         obj = BoxItem.objects.create(box=box, position=0, identifier="1234")
#         wrapper = wrapper_cls(obj)
#         self.assertEqual(
#             wrapper.href,
#             f"/admin/edc_lab/boxitem/{obj.id}/change/?"
#             f"next=edc_lab_dashboard:manage_box_listboard_url,"
#             f"box_identifier,action_name&box_identifier={box_identifier}&"
#             f"action_name=manage&",
#         )
#
#     def test_manifest_item_model_wrapper(self):
#         wrapper_cls = ManifestItemModelWrapper
#         manifest = Manifest.objects.create(
#             site=Site.objects.get_current(),
#             consignee=Consignee.objects.create(name="name"),
#             shipper=Shipper.objects.create(name="name"),
#         )
#         obj = ManifestItem.objects.create(manifest=manifest)
#         wrapper = wrapper_cls(obj)
#         self.assertEqual(
#             wrapper.href,
#             f"/admin/edc_lab/manifestitem/{obj.id}/change/?"
#             f"next=edc_lab_dashboard:manage_manifest_listboard_url&",
#         )
#
#     def test_manifest_model_wrapper(self):
#         wrapper_cls = ManifestModelWrapper
#         obj = Manifest.objects.create(
#             site=Site.objects.get_current(),
#             consignee=Consignee.objects.create(name="name"),
#             shipper=Shipper.objects.create(name="name"),
#         )
#         wrapper = wrapper_cls(obj)
#         self.assertEqual(
#             wrapper.href,
#             f"/admin/edc_lab/manifest/{obj.id}/change/?"
#             f"next=edc_lab_dashboard:manifest_listboard_url&",
#         )
#         self.assertEqual(
#             reverse("edc_lab_dashboard:manifest_listboard_url"),
#             "/edc_lab_dashboard/listboard/manifest/",
#         )
